# Results

This chapter presents findings from the benchmarking experiments
conducted to evaluate the performance of alternative image processing
libraries. The results include quantitative data on image conversion and
pixel iteration times, as well as memory consumption for each library or
combination tested. The data generated will be used to answer the
research question and support the hypotheses formulated in the previous
chapters. The benchmarking approach consisted of running two primary
tests on each library: an image conversion test that measured the time
taken to load, process, and save images, and a pixel iteration test that
recorded the time required to process every pixel in an image for a
grayscale conversion. These experiments were performed in a controlled
environment, with warm-up iterations included to reduce the impact of
initial overhead. Memory consumption was tracked alongside processing
times using BenchmarkDotNet, thereby offering a complete picture of both
speed and resource utilization.  
Before discussing the results in detail, it is important to review the
benchmarking design. In this study, each library was tested under the
same conditions: the same input image was used, a fixed number of
warm-up iterations were performed to reduce the effects of just-in-time
compilation and caching, and finally, 100 main iterations were executed
to ensure reliable statistics. For the image conversion test, the time
measured was the duration needed to load a JPEG image, convert it to
PNG, and save it back to disk. In the pixel iteration test, the focus
was on recording the time required to access and change each pixel for
producing a grayscale version of the image.

Memory diagnostics were captured concurrently, with particular attention
to allocated memory and garbage collection events. This dual approach
ensured that the results were not solely focused on speed but also took
into account the resource efficiency of each solution.

## Image Conversion Benchmark Results

The image conversion benchmark was performed using ImageSharp and
Magick.NET as well as SkiaSharp and Structure.Sketching which were the
chosen libraries in their combinations with OpenCvSharp and Emgu CV,
respectively, for the conversion task. Using the same 4k resolution
image, the benchmark measured the time taken to convert the image from
JPEG to PNG format. Comparing the results of these libraries provides
insights into their performance and efficiency in application scenarios
where rapid image conversion is required—such as real-time image
processing pipelines or high-volume batch processing environments. The
data thus answer one of the central questions regarding which library
can provide significantly faster image conversion, thereby supporting
the hypothesis discussed in earlier chapters.

ImageSharp recorded an average conversion time of approximately 2,754
milliseconds. In contrast, the combination of OpenCvSharp with SkiaSharp
delivered an average conversion time of only 539 milliseconds.
Similarly, Emgu CV integrated with Structure.Sketching achieved an
average time of 490 milliseconds, while Magick.NET registered an average
conversion time of 4,333 milliseconds.

<div id="tab:image-conversion-results">

| **Library**                       | **Warm-Up Time (ms)** | **Avg. Time Excl. Warm-Up (ms)** | **Total Time Incl. Warm-Up (ms)** |     |
|:----------------------------------|:----------------------|:---------------------------------|:----------------------------------|:----|
| **Library**                       | **Warm-Up Time (ms)** | **Avg. Time Excl. Warm-Up (ms)** | **Total Time Incl. Warm-Up (ms)** |     |
| **ImageSharp**                    |                       |                                  |                                   |     |
| **OpenCvSharp + SkiaSharp**       |                       |                                  |                                   |     |
| **Magick.NET**                    |                       |                                  |                                   |     |
| **Emgu CV + Structure.Sketching** |                       |                                  |                                   |     |
|                                   |                       |                                  |                                   |     |

The Image Conversion Benchmark Results in milliseconds, showing the
warm-up time, average time excluding warm-up, and total time including
warm-up for each library or combination.

</div>

The table
<a href="#tab:image-conversion-results" data-reference-type="ref" data-reference="tab:image-conversion-results">1.1</a>,
is the final dataset that been constructed by merging multiple Excel
files produced by the framework described in the Implementation chapter.
These results shows lightweight libraries such as SkiaSharp and
Structure.Sketching outperforming ImageSharp and Magick.NET in terms of
image conversion time. The data also reveals that Emgu CV with
Structure.Sketching is the most efficient combination for image
conversion, with the lowest average time of 490 milliseconds. On the
other hand, ImageSharp and Magick.NET are significantly slower, with
average times of 2,754 and 4,333 milliseconds, respectively.

To visually summarize these findings, Figure
<a href="#fig:image-conversion" data-reference-type="ref" data-reference="fig:image-conversion">[fig:image-conversion]</a>
presents a bar chart that depicts the conversion times across the
evaluated libraries. The graph clearly demonstrates that the conversion
times for the OpenCvSharp+SkiaSharp and Emgu CV+Structure.Sketching
combinations are positioned at the lower end of the performance
spectrum, while ImageSharp exhibits considerably higher times. A
logarithmic scale has been employed to effectively represent the
significant differences in total times—comprising both the warm-up
periods and the average conversion times. This three-color graphical
representation enables a thorough comparison of library performance in
various contexts, such as real-time image processing and batch
conversion tasks, thereby reinforcing the quantitative analysis
presented earlier.

<div class="center">

<img src="media/log_1.png" style="width:5in" alt="image" /> <span
id="fig:image-conversion"
label="fig:image-conversion">\[fig:image-conversion\]</span>

</div>

## Pixel Iteration Benchmark Results

On the other hand, the pixel iteration benchmark aimed to assess the
libraries’ abilities to process each pixel of an image. For ImageSharp,
the warm-up phase for pixel iteration took an average of 755
milliseconds, with the main iteration averaging 117.06 milliseconds per
cycle and a cumulative total of 12,461 milliseconds over 100 iterations.

<div id="tab:pixel-iteration">

| **Library**                       | **Warm-Up Time (ms)** | **Avg. Time Excl. Warm-Up (ms)** | **Total Time Incl. Warm-Up (ms)** |     |
|:----------------------------------|:----------------------|:---------------------------------|:----------------------------------|:----|
| **Library**                       | **Warm-Up Time (ms)** | **Avg. Time Excl. Warm-Up (ms)** | **Total Time Incl. Warm-Up (ms)** |     |
| **ImageSharp**                    |                       |                                  |                                   |     |
| **OpenCvSharp + SkiaSharp**       |                       |                                  |                                   |     |
| **Magick.NET**                    |                       |                                  |                                   |     |
| **Emgu CV + Structure.Sketching** |                       |                                  |                                   |     |
|                                   |                       |                                  |                                   |     |

Pixel Iteration Benchmark Results in milliseconds, showing the warm-up
time, average time excluding warm-up, and total time including warm-up
for each library or combination.

</div>

The performance landscape changed upon examining the results for
Magick.NET. This configuration recorded a warm-up time of approximately
12,149 milliseconds, and the main iterations averaged 2,054.18
milliseconds, resulting in an astronomical total of 217,567
milliseconds. As discussed earlier, OpenCvSharp and Emgu CV were chosen
in combinations with SkiaSharp and Structure.Sketching, respectively,
for the pixel iteration task. The results of these tests provide
insights into the performance of these libraries in scenarios where
pixel-level operations are required, such as image processing algorithms
or computer vision applications. The performance landscape also shifted
upon examining the results for OpenCvSharp. This configuration recorded
a warm-up time of approximately 813 milliseconds, and the main
iterations averaged 159.44 milliseconds, resulting in a total of 16,757
milliseconds. In contrast, Emgu CV delivered impressive results with a
warm-up time of 1,118 milliseconds and an average main iteration time of
118.87 milliseconds, culminating in a total of 13,005 milliseconds.

The table
<a href="#tab:pixel-iteration" data-reference-type="ref" data-reference="tab:pixel-iteration">1.2</a>
summarizes the pixel iteration benchmark results, highlighting the
warm-up and average times for each library combination. The data clearly
show that Emgu CV is the most efficient library for pixel iteration,
with the lowest average time of 118.87 milliseconds. ImageSharp and
OpenCvSharp follow closely behind, with average times of 117.06 and
159.44 milliseconds, respectively. In contrast, Magick.NET is
significantly slower, with an average time of 2,054.18
milliseconds.Graphical
<a href="#fig:pixel-iteration" data-reference-type="ref" data-reference="fig:pixel-iteration">[fig:pixel-iteration]</a>
depictions further highlight these performance differences.

<img src="media/log_2.png" style="width:5in" alt="image" /> <span
id="fig:pixel-iteration"
label="fig:pixel-iteration">\[fig:pixel-iteration\]</span>

The disparity between these figures
<a href="#fig:pixel-iteration" data-reference-type="ref" data-reference="fig:pixel-iteration">[fig:pixel-iteration]</a>
is telling. While Magick.NET excels in some aspects of image conversion,
it appears less suited for tasks involving pixel-by-pixel iteration,
given the significantly higher processing times. On the other hand, Emgu
CV and ImageSharp produce comparable main iteration times; however, when
considering the overall picture, the lower cumulative times of Emgu CV
make it a more appealing choice for pixel-level operations.

The visual comparisons elucidate that while ImageSharp and Emgu
CV+Structure.Sketching are closely matched in main iteration
performance, the excessive warm-up and overall times associated with
Magick.NET underscore its limitations for this specific task.

## Memory Benchmarking Results

In parallel with the time benchmarks, memory consumption was a critical
parameter in the evaluation. For the image conversion tasks, SkiaSharp,
as part of the OpenCvSharp+SkiaSharp configuration, exhibited the lowest
memory allocation, with values approximating 58 KB. ImageSharp, in
comparison, required about 5.67 MB, which is substantially higher. In
the context of pixel iteration, the memory profiles were similarly
divergent. ImageSharp was extremely efficient in this regard, consuming
roughly 20 KB on average, whereas Emgu CV + Structure.Sketching, that
performed exceptionally well in terms of speed for pixel iteration, in
memory terms, was less efficient. It consumed around 170 MB of memory,
which is significantly higher than the other libraries tested.
SkiaSharp,

<div id="tab:memory-results-image-conversion">

| **Library**    | **Allocated Memory** | **Gen0/Gen1/Gen2 Collections** |     |     |
|:---------------|:---------------------|:-------------------------------|:----|:----|
| **Library**    | **Allocated Memory** | **Gen0/Gen1/Gen2 Collections** |     |     |
| **EmguCV**     | MB (712 bytes)       | \- / - / -                     |     |     |
| **ImageSharp** | MB (5,805.41 KB)     | ,000 / 1,000 / 1,000           |     |     |
| **SkiaSharp**  | MB (58,864 bytes)    | \- / - / -                     |     |     |
|                |                      |                                |     |     |

Memory benchmarking results for the image conversion task, detailing the
allocated memory (in MB) along with the associated Gen0, Gen1, and Gen2
garbage collection counts.

</div>

The table
<a href="#tab:memory-results-image-conversion" data-reference-type="ref" data-reference="tab:memory-results-image-conversion">1.3</a>
summarizes the memory benchmarking results for image conversion. It is
evident that ImageSharp has the highest memory allocation, with
approximately 5.67 MB, while SkiaSharp has the lowest. Emgu CV falls in
between, with a memory allocation of 0.00068 MB. These figures provide a
clear indication of the memory efficiency of each library for image
conversion tasks. Garbage collection counts are also included to provide
additional context on the memory management behavior of each library.
Gen0, Gen1, and Gen2 collections means the number of times each
generation was collected during the benchmarking process. These metrics
are essential for understanding how each library manages memory and how
it impacts performance.

The large memory footprint of Emgu CV during pixel iteration is a
noteworthy trade-off. While its performance in terms of speed is
excellent, the high memory consumption must be considered when deploying
the solution in memory-constrained environments. The benchmarking data
collected here is critical because it provides a balanced view—speed
alone does not define an optimal library, but rather the ratio of
processing time to memory usage does. For a clear summary of these
findings, table
<a href="#tab:memory-results-pixel-iteration" data-reference-type="ref" data-reference="tab:memory-results-pixel-iteration">1.4</a>
provides a concise overview of the memory metrics for each library
configuration.

<div id="tab:memory-results-pixel-iteration">

| **Library**    | **Allocated Memory**   | **Gen0/Gen1/Gen2 Collections** |     |     |
|:---------------|:-----------------------|:-------------------------------|:----|:----|
| **Library**    | **Allocated Memory**   | **Gen0/Gen1/Gen2 Collections** |     |     |
| **EmguCV**     | MB (177,976,185 bytes) | ,142 / 1,571 / 1,571           |     |     |
| **ImageSharp** | MB (20.26 KB)          | \- / - / -                     |     |     |
| **SkiaSharp**  | MB (403,300,552 bytes) | / - / -                        |     |     |
|                |                        |                                |     |     |

Memory Benchmarking Results for Pixel Iteration Task, detailing the
allocated memory (in MB) along with the associated Gen0, Gen1, and Gen2
garbage collection counts.

</div>

The table
<a href="#tab:memory-results-pixel-iteration" data-reference-type="ref" data-reference="tab:memory-results-pixel-iteration">1.4</a>
indicates that while SkiaSharp has the highest memory allocation for
pixel iteration of approximately 384 MB, ImageSharp is the most
memory-efficient, with a memory allocation of 0.01932 MB. Emgu CV falls
in between, with a memory allocation of 170 MB. These figures provide a
clear indication of the memory efficiency of each library for pixel
iteration tasks. Garbage collection counts are also included to provide
additional context on the memory management behavior of each library.
Gen0, Gen1, and Gen2 collections means the number of times each
generation was collected during the benchmarking process. This means
that the garbage collector had to run 33,142 times for Gen0, 1,571 times
for Gen1, and 1,571 times for Gen2.

## Analysis and Interpretation of Results

As the final benchmarking results were collected and plotted, the
emerging trends provided critical insights into the efficiency of
various image processing libraries. The raw numerical data from the
benchmarking suite provided an answer to the research question, but a
deeper interpretation of these results allowed refinement of the
understanding of the trade-offs and strengths of each alternative. This
section explores the relationship between speed and memory usage,
compares the empirical findings with theoretical expectations, and
discusses the implications for real-world applications.

### Comparison of Performance Trends

The performance hierarchy observed in the benchmarking results closely
aligns with expectations based on each library’s internal architecture.
Libraries such as OpenCvSharp and Emgu CV, both built upon OpenCV’s
optimized C++ backend, showcased superior execution times for pixel
iteration tasks. This efficiency is largely attributed to OpenCV’s
reliance on low-level SIMD (Single Instruction, Multiple Data)
optimizations and hardware-accelerated processing paths.

Conversely, ImageSharp—despite its clean API and pure C#
implementation—demonstrated significantly higher processing times,
reinforcing the general principle that managed code introduces overhead
compared to native libraries. In memory-constrained environments, the
trade-off between speed and memory usage should be carefully considered
as ImageSharp’s memory efficiency may outweigh its slower execution
times. ImageSharp remains a viable option for applications prioritizing
ease of use and portability over raw performance or in scenarios where
memory conservation is critical.

Magick.NET, though powerful and highly flexible in terms of format
support, performed noticeably worse in pixel iteration tasks. This
result was somewhat anticipated due to the internal structure of
ImageMagick, which prioritizes format conversions and high-quality
rendering over raw pixel access speed. The excessive processing times
observed in the Magick.NET pixel iteration benchmark further support the
hypothesis that it is not optimized for this type of operation. However,
its range of features and extensive format support make it a compelling
choice for applications requiring advanced image processing
capabilities.

The trends in memory consumption were particularly revealing. In the
image conversion test, SkiaSharp exhibited the lowest memory usage, also
demonstrating competitive processing times. This result is consistent
with SkiaSharp’s reputation for being lightweight and efficient, making
it an excellent choice for applications need high performance and low
memory overhead. In the pixel iteration test, Emgu CV memory usage was
significantly higher than ImageSharp, highlighting the trade-off between
speed and memory efficiency. This finding underscores the importance of
selecting the right library based on the specific requirements of the
application. This observation is consistent with Emgu CV’s underlying
OpenCV core, which relies on large temporary buffers and matrix
structures for intermediate computations. In contrast, ImageSharp
demonstrated exceptional memory efficiency during pixel iteration but
was significantly slower, suggesting that its architecture prioritizes
memory conservation over execution speed.

### Trade-Offs Between Speed and Memory Usage

The relationship between speed and memory consumption is a recurring
theme in performance optimization. Results underscore that achieving
optimal speed often comes at the cost of increased memory usage. Emgu
CV+Structure.Sketching exemplifies this trade-off: while its pixel
iteration speed was among the best recorded, it consumed significantly
more RAM than ImageSharp.

The implications of these trade-offs depend heavily on the intended
application. For environments where processing speed is paramount—such
as real-time video processing or AI-powered image enhancement—Emgu CV’s
increased memory footprint may be an acceptable compromise. However, in
resource-constrained applications (e.g., embedded systems, mobile
devices, or cloud-based deployments with strict memory limits), a
lower-memory alternative like ImageSharp may be more suitable despite
its lower speed.

<div id="tab:speed-memory-trade-offs">

| **Library**   | **Task**         | **Speed** | **Memory Usage** |
|:--------------|:-----------------|:----------|:-----------------|
| **Library**   | **Task**         | **Speed** | **Memory Usage** |
|               | Image Conversion | Slow      | Low              |
|               | Pixel Iteration  | Fast      | Low              |
| **Emgu CV**   | Pixel Iteration  | Fast      | High             |
| **SkiaSharp** | Image Conversion | Fast      | Low              |
|               |                  |           |                  |

Table of Speed and Memory Trade-Offs for Image Processing Libraries, the
fast/slow and high/low are relative to the other libraries.

</div>

One particularly interesting finding was that OpenCvSharp+SkiaSharp
consistently delivered both high speed and low memory usage for image
conversion. This anomaly suggests that this combination strikes an
optimal balance, leveraging OpenCV’s native optimizations while
maintaining a lightweight footprint in memory. The fact that this hybrid
approach outperformed even standalone OpenCV libraries further supports
the notion that combining high-performance native libraries with
efficient rendering engines can yield superior results.

## Summary

The benchmarking results provide a comprehensive overview of the
performance and efficiency of the image processing libraries tested. The
data clearly show that Emgu CV + Structure.Sketching is the most
efficient combination for image conversion, with the lowest average time
of 490 milliseconds. In contrast, ImageSharp and Magick.NET are
significantly slower, with average times of 2,754 and 4,333
milliseconds, respectively. For pixel iteration, Emgu
CV+Structure.Sketching is again the most efficient, with the lowest
average time of 118.87 milliseconds. ImageSharp and
OpenCvSharp+SkiaSharp follow closely behind, with average times of
117.06 and 159.44 milliseconds, respectively. In contrast, Magick.NET is
significantly slower, with an average time of 2,054.18 milliseconds. The
memory benchmarking results further highlight the efficiency of
ImageSharp and SkiaSharp in terms of memory consumption, with Emgu CV
exhibiting higher memory usage. Developers can use these findings to
select the most suitable library for their particular needs based on
their specific requirements and constraints regarding speed and resource
utilization.
