# Appendix

#### Evaluation of Image Processing Libraries

This appendix provides a detailed analysis of various image processing
libraries considered for the implementation phase of this thesis. Each
library is evaluated based on key technical criteria, licensing
considerations, and integration requirements.

<div id="openimageio-oiio">

#### 1. OpenImageIO (OIIO)

</div>

-   **Type**: Open-source

-   **Key Features**: Supports numerous image formats, extensive image
    processing functionalities

-   **Licensing**: Free (BSD license)

-   **Performance**: Known for high performance in professional
    pipelines

-   **Integration Effort**: Moderate, requires familiarity with C++ or
    Python bindings

-   **Community and Support**: Active community, well-documented

**There might be a need to use P/Invoke or shell out to OIIO
command-line utilities if there’s no direct C# wrapper.**

| **Feature Category**                   | **Supported by OIIO**                                                                                                                                                                                                                                                           | **Not Natively Supported / Requires Custom Implementation**                                                                                                         |
|:---------------------------------------|:--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:--------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by OIIO**                                                                                                                                                                                                                                                           | **Not Natively Supported / Requires Custom Implementation**                                                                                                         |
| **Image Loading and Creation**         | \- **ImageInput:open**: Opens an image file for reading. - **ImageInput:read_image**: Reads image data. - **ImageOutput:create**: Creates a new image file for writing. - **ImageOutput:write_image**: Writes the image data. - **ImageBuf**: Can create an empty image buffer. | \- Asynchronous image loading (no equivalent to **Image.LoadAsync**).                                                                                               |
| **Image Processing and Manipulation**  | \- Basic pixel manipulation through scanlines/tiles. - **ImageBufAlgo** provides some algorithms for manipulation.                                                                                                                                                              | \- No built-in functions for **Clone**, **Mutate**, **Resize**, **Grayscale**. - Requires external libraries (e.g., OpenCV) or custom code for advanced processing. |
| **Pixel Formats**                      | \- Supports various pixel formats including RGBA, RGB, L (grayscale), YUV, etc. - **ImageBufAlgo:colorconvert** allows conversion between formats.                                                                                                                              | \- Specific formats like **Byte4**, **YCbCr** may require manual handling.                                                                                          |
| **Pixel Access and Manipulation**      | \- Provides pixel access through **ImageInput:read_scanline** and **ImageInput:read_tile**. - Can manipulate pixels by reading and writing scanlines/tiles.                                                                                                                     | \- No direct method like **ProcessPixelRows**; manual processing required.                                                                                          |
| **Image Metadata and Conversion**      | \- **ImageSpec**: Handles image metadata. - **ImageBuf**: Manages pixel data and metadata. - **ImageBufAlgo**: Offers conversion algorithms.                                                                                                                                    | \- Handling complex metadata and conversions might require custom implementation depending on needs.                                                                |
| **Creating and Disposing Instances**   | \- **ImageBuf**: Creates and manages image instances. - Resources automatically managed in C++ (via destructors).                                                                                                                                                               | \- Explicit disposal may be needed for resource-intensive operations, especially in languages without automatic garbage collection.                                 |
| **Cropping and Resizing**              | \- **ImageBufAlgo:crop**: Crops images. - **ImageBufAlgo:resize**: Resizes images with various techniques.                                                                                                                                                                      | \- No direct equivalent to **Mutate** for fluent transformations.                                                                                                   |
| **Encoding Images in Various Formats** | \- Supports encoding in multiple formats (BMP, JPEG, PNG, TIFF, **WebP**, etc.) via **ImageOutput**.                                                                                                                                                                            | \- none.                                                                                                                                                            |
| **Composing Image Layers**             | \- **ImageBufAlgo:paste**: Combines image layers or tiles.                                                                                                                                                                                                                      | \- No built-in methods equivalent to ImageSharp’s **Stitch** method.                                                                                                |
| **Resampling Methods**                 | \- **ImageBufAlgo:resample**: Provides resampling techniques.                                                                                                                                                                                                                   | \- Lacks a direct equivalent to the **IResampler** interface; resampling management is manual.                                                                      |
| **Saving the Image**                   | \- **ImageOutput:write_image**: Saves images to a file.                                                                                                                                                                                                                         | \- No asynchronous saving; requires standard async techniques for implementation.                                                                                   |

<div id="skiasharp">

#### 2. SkiaSharp

</div>

-   **Type**: Open-source

-   **Key Features**: High-performance 2D graphics library, various
    image processing tasks

-   **Licensing**: Free (MIT License)

-   **Performance**: High performance, optimized for cross-platform use

-   **Integration Effort**: Easy, seamless integration with .NET Core

-   **Community and Support**: Active community, extensive documentation
    and examples

-   **Restricted Countries contributors:** Need a check!

| **Feature Category**                   | **Supported by SkiaSharp**                                                                                                                                                                                                                                                                                                             | **Not Natively Supported / Requires Custom Implementation**                                          |
|:---------------------------------------|:---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:-----------------------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by SkiaSharp**                                                                                                                                                                                                                                                                                                             | **Not Natively Supported / Requires Custom Implementation**                                          |
| **Image Loading and Creation**         | \- **SKBitmap.Decode**: Loads images from byte arrays or streams. - **SKBitmap**: Creates new bitmaps with specified dimensions and color types. - **SKImage.FromBitmap**: Creates an image from a bitmap. - **SKImage.FromEncodedData**: Loads images from encoded data.                                                              | \- No built-in asynchronous image loading (no equivalent to **Image.LoadAsync**).                    |
| **Image Processing and Manipulation**  | \- **SKBitmap.Copy**: Clones the bitmap. - **SKBitmap.Resize**: Resizes bitmaps with various filtering modes. - **SKBitmap.ExtractSubset**: Crops bitmaps. - **SKImage.FilterImage**: Applies filters like grayscale or other color transformations. - **SKColorFilter.CreateColorMatrix**: Provides additional color transformations. | \- No high-level fluent API like ImageSharp’s **Mutate** for chaining multiple operations.           |
| **Pixel Formats**                      | \- Supports various pixel formats like **SKColorType.Rgba8888**, **SKColorType.Bgra8888**, **SKColorType.Gray8**, etc. - **SKColorSpace**: Manages color space conversions.                                                                                                                                                            | \- Specific formats like **Byte4**, **YCbCr** may require custom conversion.                         |
| **Pixel Access and Manipulation**      | \- **SKBitmap.GetPixel** / **SetPixel**: Accesses and sets individual pixels. - **SKBitmap.Pixels**: Provides access to the pixel data for bulk manipulation.                                                                                                                                                                          | \- No direct method like **ProcessPixelRows**; manual processing of pixel rows is required.          |
| **Image Metadata and Conversion**      | \- **SKImageInfo**: Manages basic image properties such as dimensions and color type. - **SKImage.Encode**: Converts images into various formats like PNG, JPEG, **WebP**, etc.                                                                                                                                                        | \- Limited support for complex metadata handling compared to ImageSharp and OIIO.                    |
| **Creating and Disposing Instances**   | \- **SKBitmap** and **SKImage**: Create and manage image instances. - Proper resource management using the **Dispose** method is necessary to free resources.                                                                                                                                                                          | \- No direct equivalent to **Image\<Rgba32>**; pixel format and dimensions must be managed manually. |
| **Cropping and Resizing**              | \- **SKBitmap.ExtractSubset**: Crops images. - **SKBitmap.Resize**: Resizes images with different resampling techniques.                                                                                                                                                                                                               | \- No built-in equivalent to ImageSharp’s **Mutate** method for complex transformations.             |
| **Encoding Images in Various Formats** | \- **SKImage.Encode**: Encodes images in multiple formats (e.g., PNG, JPEG, BMP, **WebP**).                                                                                                                                                                                                                                            | \- Custom handling might be needed for less common formats.                                          |
| **Composing Image Layers**             | \- **SKCanvas.DrawBitmap**: Composes images by drawing one bitmap onto another. - **SKPicture**: Records a sequence of drawing commands for later playback and compositing.                                                                                                                                                            | \- No built-in method equivalent to ImageSharp’s **Stitch** for seamless image stitching.            |
| **Resampling Methods**                 | \- **SKBitmap.Resize**: Provides resampling techniques during resizing operations.                                                                                                                                                                                                                                                     | \- No direct equivalent to **IResampler** interface; resampling techniques are more basic.           |
| **Saving the Image**                   | \- **SKImage.Encode**: Saves images to a stream or byte array in the desired format.                                                                                                                                                                                                                                                   | \- No built-in asynchronous saving method; async saving requires custom implementation.              |

<div id="magick.net">

#### 3. Magick.NET

</div>

-   **Type**: Open-source

-   **Key Features**: .NET wrapper for ImageMagick, extensive image
    manipulation capabilities

-   **Licensing**: Free (Apache 2.0 License)

-   **Performance**: Excellent for complex image processing

-   **Integration Effort**: Moderate, straightforward API

-   **Community and Support**: Large user base, comprehensive
    documentation

-   **Restricted Countries contributors:** No

| **Feature Category**                   | **Supported by Magick.NET**                                                                                                                                                                                                                                                                                                                                                                                                                                                                              | **Not Natively Supported / Requires Custom Implementation**                                                                           |
|:---------------------------------------|:---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:--------------------------------------------------------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by Magick.NET**                                                                                                                                                                                                                                                                                                                                                                                                                                                                              | **Not Natively Supported / Requires Custom Implementation**                                                                           |
| **Image Loading and Creation**         | \- **MagickImage**: Loads images from byte arrays, files, or streams. - **MagickImageCollection**: Handles multiple images, useful for formats like GIFs. - **MagickImage**: Creates new images with specified dimensions and colors.                                                                                                                                                                                                                                                                    | \- No built-in asynchronous image loading (no equivalent to **Image.LoadAsync**).                                                     |
| **Image Processing and Manipulation**  | \- **MagickImage.Clone**: Clones the image. - **MagickImage.Resize**: Resizes images with various filtering and resampling options. - **MagickImage.Crop**: Crops images. - **MagickImage.Resize**: Resizes images with customizable resampling options. - **MagickImage.Grayscale**: Converts images to grayscale. - **MagickImage.ColorSpace**: Converts between different color spaces. - **MagickImage.Rotate**, **MagickImage.Flip**, **MagickImage.Flop**: Performs various image transformations. | \- Magick.NET supports extensive image processing, similar to ImageSharp’s **Mutate** method. Custom implementation is rarely needed. |
| **Pixel Formats**                      | \- Supports a wide range of pixel formats, including RGBA, RGB, Gray, CMYK, and more. - **MagickColor**: Handles color conversions and supports various color profiles.                                                                                                                                                                                                                                                                                                                                  | \- Fully supports advanced pixel formats and color management, so custom implementation is minimal.                                   |
| **Pixel Access and Manipulation**      | \- **MagickImage.GetPixels**: Provides access to individual pixels or pixel regions. - **MagickImage.SetPixels**: Sets individual pixels. - **MagickImage.ToByteArray**: Converts pixel data to a byte array.                                                                                                                                                                                                                                                                                            | \- No direct method like **ProcessPixelRows**; however, pixel manipulation is flexible and powerful.                                  |
| **Image Metadata and Conversion**      | \- **MagickImage.Attribute**: Accesses and manipulates image metadata such as EXIF, IPTC, and XMP. - **MagickImage.Format**: Converts images to various formats.                                                                                                                                                                                                                                                                                                                                         | \- Magick.NET offers comprehensive metadata handling and format conversion.                                                           |
| **Creating and Disposing Instances**   | \- **MagickImage**: Creates and manages image instances. - Proper resource management using the **Dispose** method is necessary to free resources. - **MagickImageCollection**: Manages multiple image instances, useful for animations or multi-layer images.                                                                                                                                                                                                                                           | \- Fully supports instance creation and disposal, with comprehensive memory management.                                               |
| **Cropping and Resizing**              | \- **MagickImage.Crop**: Crops images. - **MagickImage.Resize**: Resizes images with customizable resampling options. - **MagickImage.AdaptiveResize**: Provides advanced resizing techniques.                                                                                                                                                                                                                                                                                                           | \- Fully supports cropping and resizing with advanced options, no need for custom implementation.                                     |
| **Encoding Images in Various Formats** | \- **MagickImage.Write**: Encodes images in a wide array of formats including PNG, JPEG, TIFF, BMP, GIF, **WebP**, and more. - **MagickImage.Format**: Specifies the output format.                                                                                                                                                                                                                                                                                                                      | \- Supports a wider range of formats than ImageSharp, with built-in encoding capabilities.                                            |
| **Composing Image Layers**             | \- **MagickImage.Composite**: Composes one image over another. - **MagickImage.Mosaic**: Combines multiple images into a mosaic. - **MagickImageCollection**: Handles layering for complex compositions.                                                                                                                                                                                                                                                                                                 | \- Fully supports complex image compositions, with built-in methods for layering and merging.                                         |
| **Resampling Methods**                 | \- **MagickImage.Resample**: Provides advanced resampling methods and filtering options. - **MagickImage.AdaptiveResize**: Offers specialized resampling techniques.                                                                                                                                                                                                                                                                                                                                     | \- Extensive support for resampling, surpassing basic needs and requiring no custom implementation.                                   |
| **Saving the Image**                   | \- **MagickImage.Write**: Saves images to files, streams, or byte arrays in the desired format. - **MagickImage.Save**: Provides simple saving options.                                                                                                                                                                                                                                                                                                                                                  | \- No built-in asynchronous saving method; async saving requires custom implementation.                                               |

<div id="emgu-cv">

#### 4. Emgu CV

</div>

-   **Type**: Open-source

-   **Key Features**: .NET wrapper for OpenCV, robust image processing
    and computer vision

-   **Licensing**: 799$ (for version 4, with additional costs for
    upgrades).

-   **Performance**: High performance, suitable for advanced computer
    vision tasks

-   **Integration Effort**: Moderate to high, depending on complexity of
    use

-   **Community and Support**: Active community, extensive tutorials

-   **Restricted Countries contributors:** Need a check

| **Feature Category**                   | **Supported by Emgu CV**                                                                                                                                                                                                                                                                                                                                                      | **Not Natively Supported / Requires Custom Implementation**                                                               |
|:---------------------------------------|:------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:--------------------------------------------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by Emgu CV**                                                                                                                                                                                                                                                                                                                                                      | **Not Natively Supported / Requires Custom Implementation**                                                               |
| **Image Loading and Creation**         | \- **CvInvoke.Imread**: Loads an image from a file. - **CvInvoke.Imdecode**: Loads an image from a byte array. - **Mat**: Creates a new image with specified dimensions and type. - **Image\<TColor, TDepth>**: Generic class for creating images with specific color and depth.                                                                                              | \- No built-in asynchronous image loading (no equivalent to **Image.LoadAsync**).                                         |
| **Image Processing and Manipulation**  | \- **Mat.Clone**: Clones the image matrix. - **CvInvoke.Resize**: Resizes the image with various interpolation methods. - **CvInvoke.CvtColor**: Converts the image to grayscale or other color spaces. - **CvInvoke.Rotate**: Rotates the image. - **CvInvoke.Flip**: Flips the image vertically or horizontally. - **CvInvoke.WarpAffine**: Applies affine transformations. | \- Emgu CV provides extensive support for image processing operations, similar to ImageSharp’s **Mutate** method.         |
| **Pixel Formats**                      | \- Supports various pixel formats including BGR, RGBA, RGB, and Gray. - **Mat.Depth** and **Mat.NumberOfChannels**: Specify pixel depth and channels. - **Image\<TColor, TDepth>**: Allows pixel data manipulation in a type-safe manner.                                                                                                                                     | \- Default BGR format might differ from ImageSharp’s RGBA, requiring format conversion in some cases.                     |
| **Pixel Access and Manipulation**      | \- **Mat.GetData**: Accesses individual pixels or pixel regions. - **Mat.SetTo**: Sets individual pixels or regions with specific values. - **Image\<TColor, TDepth>.Data**: Provides access to pixel data.                                                                                                                                                                   | \- Pixel manipulation is supported, but the approach differs from ImageSharp’s more abstracted methods.                   |
| **Image Metadata and Conversion**      | \- **CvInvoke.Imencode**: Converts the image to various formats (PNG, JPEG, etc.). - **CvInvoke.Imwrite**: Saves images to files.                                                                                                                                                                                                                                             | \- Does not extensively handle metadata like EXIF or IPTC, focusing more on basic properties and format conversion.       |
| **Creating and Disposing Instances**   | \- **Mat**: Can be used to create empty images, with proper disposal via **Dispose** to free resources. - **Image\<TColor, TDepth>**: Manages images with type safety.                                                                                                                                                                                                        | \- Fully supports instance creation and disposal, requiring careful memory management due to OpenCV’s low-level handling. |
| **Cropping and Resizing**              | \- **CvInvoke.GetRectSubPix**: Crops the image. - **CvInvoke.Resize**: Resizes the image with advanced interpolation options.                                                                                                                                                                                                                                                 | \- Emgu CV supports cropping and resizing extensively, similar to ImageSharp’s **Mutate** method.                         |
| **Encoding Images in Various Formats** | \- **CvInvoke.Imwrite**: Saves images in formats like PNG, JPEG, BMP, **WebP**, etc. - **CvInvoke.Imencode**: Encodes images for various formats and uses.                                                                                                                                                                                                                    | \- none.                                                                                                                  |
| **Composing Image Layers**             | \- **CvInvoke.AddWeighted**: Blends two images together, allowing for composition. - **CvInvoke.CopyMakeBorder**: Combines images into a larger canvas.                                                                                                                                                                                                                       | \- Supports basic layer composition, though complex operations may require additional code or OpenCV functions.           |
| **Resampling Methods**                 | \- **CvInvoke.Resize**: Provides multiple resampling methods (linear, cubic, nearest-neighbor).                                                                                                                                                                                                                                                                               | \- Resampling techniques are fully supported, similar to ImageSharp’s capabilities.                                       |
| **Saving the Image**                   | \- **CvInvoke.Imwrite**: Saves images to files in the desired format.                                                                                                                                                                                                                                                                                                         | \- No built-in asynchronous saving method; async operations require custom implementation.                                |

<div id="magicscaler">

#### 5. MagicScaler

</div>

-   **Type**: Open-source

-   **Key Features**: High-performance image processing, optimized for
    resizing

-   **Licensing**: Free

-   **Performance**: Excellent for image resizing with high quality

-   **Integration Effort**: Easy, designed for high-performance
    scenarios

-   **Community and Support**: Active, good documentation

-   **Restricted Countries contributors:** No

| **Feature Category**                   | **Supported by MagicScaler**                                                                                                                                                                                  | **Not Natively Supported / Requires Custom Implementation**                                                                                                           |
|:---------------------------------------|:--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:----------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by MagicScaler**                                                                                                                                                                                  | **Not Natively Supported / Requires Custom Implementation**                                                                                                           |
| **Image Loading and Creation**         | \- **MagicImageProcessor.ProcessImage**: Loads and processes an image from a file, stream, or byte array. - **ImageFileInfo**: Provides basic details about the image without fully loading it into memory.   | \- No direct method for creating new images from scratch. - Lacks asynchronous methods like **Image.LoadAsync** in ImageSharp.                                        |
| **Image Processing and Manipulation**  | \- **MagicImageProcessor.ProcessImage**: Supports resizing, cropping, and color adjustments. - **ProcessImageSettings**: Allows specifying transformations such as resizing, cropping, and color adjustments. | \- Lacks complex manipulation like cloning, rotating, flipping, or direct pixel manipulation. - Does not support advanced image editing features found in ImageSharp. |
| **Pixel Formats**                      | \- Supports various pixel formats including RGBA, RGB, Gray, etc. - Automatically handles color space conversions.                                                                                            | \- Handling of pixel formats is abstracted, with no direct pixel manipulation like in ImageSharp.                                                                     |
| **Pixel Access and Manipulation**      | \- Provides high-level processing but no direct pixel access.                                                                                                                                                 | \- No direct access or manipulation of individual pixels, unlike ImageSharp’s **ProcessPixelRows**.                                                                   |
| **Image Metadata and Conversion**      | \- Handles basic image properties and metadata. - Supports conversion between different image formats.                                                                                                        | \- Limited metadata handling compared to libraries like ImageSharp, focusing more on image processing efficiency.                                                     |
| **Creating and Disposing Instances**   | \- Focuses on processing existing images rather than creating new ones. - Managed disposal of resources.                                                                                                      | \- Does not support creating images from scratch. - No manual instance creation like in ImageSharp.                                                                   |
| **Cropping and Resizing**              | \- **ProcessImageSettings.Crop**: Specifies cropping options. - **ProcessImageSettings.Width and Height**: Specifies resizing dimensions. - Uses high-quality resampling algorithms for resizing.             | \- Cropping and resizing are part of the processing pipeline, not standalone operations.                                                                              |
| **Encoding Images in Various Formats** | \- Supports encoding images into formats like PNG, JPEG, BMP, **WebP**, etc. - Can control compression, quality, and other encoding parameters.                                                               | \- Lacks format-specific customization that ImageSharp provides through various encoders.                                                                             |
| **Composing Image Layers**             | \- Not supported; focuses on single-image processing.                                                                                                                                                         | \- Lacks capabilities for composing or layering multiple images, unlike ImageSharp’s **Stitch** or **DrawImage**.                                                     |
| **Resampling Methods**                 | \- Provides high-quality resampling techniques for resizing. - **ProcessImageSettings.Interpolation**: Allows specifying the interpolation method.                                                            | \- Resampling is integrated into the processing pipeline, without a direct interface like **IResampler** in ImageSharp.                                               |
| **Saving the Image**                   | \- **MagicImageProcessor.ProcessImage**: Saves the processed image to a file, stream, or byte array.                                                                                                          | \- No asynchronous saving method, with all operations handled synchronously.                                                                                          |

<div id="simpleitk">

#### 6. SimpleITK

</div>

-   **Type**: Open-source

-   **Key Features**: Interface to the Insight Toolkit (ITK), simplifies
    complex image analysis

-   **Licensing**: Free

-   **Performance**: Suitable for medical and scientific image
    processing

-   **Integration Effort**: Moderate, specialized usage

-   **Community and Support**: Good community support, extensive
    resources

-   **Restricted Countries contributors:** Need a check!

| **Feature Category**                   | **Supported by SimpleITK**                                                                                                                                                                                                                          | **Not Natively Supported / Requires Custom Implementation**                                                                                                                   |
|:---------------------------------------|:----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by SimpleITK**                                                                                                                                                                                                                          | **Not Natively Supported / Requires Custom Implementation**                                                                                                                   |
| **Image Loading and Creation**         | \- **sitk.ReadImage**: Loads an image from a file. - **sitk.Image**: Creates new images with specified dimensions and pixel types. - **sitk.ImportImageFilter**: Loads image data from NumPy arrays.                                                | \- No asynchronous loading methods like **Image.LoadAsync**. - More focused on scientific image formats and lacks direct support for common web image formats.                |
| **Image Processing and Manipulation**  | \- **sitk.Clone**, **sitk.Resample**, **sitk.Cast**: For cloning, resizing, and pixel type conversion. - **sitk.Transform**, **sitk.Crop**: Supports geometric transformations and cropping. - **sitk.Grayscale**: Converts the image to grayscale. | \- Lacks the high-level abstraction of ImageSharp’s **Mutate** for chaining operations. - More tailored to medical image processing than artistic or graphical manipulations. |
| **Pixel Formats**                      | \- Supports a variety of formats, including scalar, vector, and label images. - Automatic pixel type conversions.                                                                                                                                   | \- Limited in handling non-medical specific color spaces and formats compared to ImageSharp.                                                                                  |
| **Pixel Access and Manipulation**      | \- **sitk.GetPixel/SetPixel**: Access and manipulate individual pixels. - Full image data accessible via NumPy arrays for batch processing.                                                                                                         | \- Less intuitive pixel manipulation compared to ImageSharp’s **ProcessPixelRows**. - Focus on scientific data rather than general-purpose image formats.                     |
| **Image Metadata and Conversion**      | \- **sitk.Image**: Handles metadata such as image origin, spacing, and direction. - **sitk.Cast**: Converts images to various pixel types.                                                                                                          | \- Limited advanced metadata handling, focusing on medical image properties.                                                                                                  |
| **Creating and Disposing Instances**   | \- **sitk.Image**: Creates empty images. - Automatic resource management with Python’s garbage collector.                                                                                                                                           | \- Does not offer the fine-grained control over image creation seen in ImageSharp.                                                                                            |
| **Cropping and Resizing**              | \- **sitk.Crop**: Crops the image. - **sitk.Resample**: Provides resizing with multiple interpolation methods.                                                                                                                                      | \- Requires more manual setup compared to the intuitive API of ImageSharp’s **Mutate**.                                                                                       |
| **Encoding Images in Various Formats** | \- **sitk.WriteImage**: Supports encoding and saving images in formats like PNG, JPEG, and TIFF.                                                                                                                                                    | \- Focuses more on medical image formats, and does not natively support **WebP**.                                                                                             |
| **Composing Image Layers**             | \- Can handle multi-channel images, simulating layer composition. - **sitk.LabelOverlay**: Can overlay labels on grayscale images.                                                                                                                  | \- Lacks direct support for layer-based compositions found in ImageSharp. - Limited artistic composition tools.                                                               |
| **Resampling Methods**                 | \- **sitk.Resample**: Offers various resampling methods with advanced interpolation options.                                                                                                                                                        | \- Resampling is powerful but less user-friendly than ImageSharp’s high-level options.                                                                                        |
| **Saving the Image**                   | \- **sitk.WriteImage**: Saves images to files in various formats. - Supports scientific formats like NIfTI, DICOM.                                                                                                                                  | \- Limited optimization for modern web formats compared to ImageSharp.                                                                                                        |

<div id="structure.sketching">

#### 7. Structure.Sketching

</div>

-   **Type**: Open-source

-   **Key Features**: Image transformations, filters, and format support

-   **Licensing**: Free (Apache 2.0 License)

-   **Performance**: Competitive, with various resampling filters and
    transformations

-   **Integration Effort**: Easy to moderate, supports .NET Core and
    .NET Framework

-   **Community and Support**: Growing community, good initial
    documentation

-   **Restricted Countries contributors:** No

| **Feature Category**                   | **Supported by Image Package**                                                                                                                                                                                                                                                                                                                                                                | **Not Natively Supported / Requires Custom Implementation**                                                                                                                                                                                                                               |
|:---------------------------------------|:----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by Image Package**                                                                                                                                                                                                                                                                                                                                                                | **Not Natively Supported / Requires Custom Implementation**                                                                                                                                                                                                                               |
| **Image Loading and Creation**         | \- **png.loadPNG**: Typically read an image file and decode it using the appropriate decoder from a format-specific sub-package. - **image.NewRGBA**: Can create new images using the **image** package, which provides several types like **image.RGBA**, **image.NRGBA**, **image.Gray**, etc. - **png.Encode**: Can save it using an encoder from the appropriate format-specific package. | \- Limited support for creating images from scratch in more diverse formats. - Requires third-party libraries for formats like **WebP**. - No inherent support for asynchronous operations, can achieve asynchronous behavior by running image-related operations in separate goroutines. |
| **Image Processing and Manipulation**  | \- **Clone**, **Resize**, **ConvertToGrayscale**: Supports advanced image processing tasks. - **Transformations**, **Annotations**: Extensive transformations and annotation capabilities. - **Image Enhancement**: High-level enhancement tools for noise reduction, contrast adjustment, etc.                                                                                               | \- Requires custom code implementation.                                                                                                                                                                                                                                                   |
| **Pixel Formats**                      | \- **image.RGB**: Represents a color image with 8-bit RGBA values per pixel. - **image.Gray**: Represents a grayscale image with 8-bit gray values per pixel. - **image.YCbCr**: Represents a color image using the Y’CbCr color model, typically used in JPEG images.                                                                                                                        | \- Basic support for different image formats but does not explicitly handle pixel formats like image processing libraries.                                                                                                                                                                |
| **Pixel Access and Manipulation**      | \- No direct methods like **getpixel**, **setpixel** or **RasterImageData**.                                                                                                                                                                                                                                                                                                                  | \- No direct pixel access, unlike ImageSharp’s **ProcessPixelRows**.                                                                                                                                                                                                                      |
| **Image Metadata and Conversion**      | \- Does not support comprehensive metadata handling directly. - Focuses on image manipulation and format conversion but lacks built-in tools for managing or accessing metadata like EXIF data or other detailed image properties.                                                                                                                                                            | \- Lack of support requires custom implementations.                                                                                                                                                                                                                                       |
| **Creating and Disposing Instances**   | \- Use functions from the **image** package along with specific image formats (like **image/png**, **image/jpeg**). - Go’s garbage collector handles memory management, so no need to manually dispose of images.                                                                                                                                                                             | \- Does not directly provide **CreateImage** or **Dispose** functions as in other libraries or languages with more explicit image management.                                                                                                                                             |
| **Cropping and Resizing**              | \- Does not include built-in support for more advanced operations like cropping and resizing directly.                                                                                                                                                                                                                                                                                        | \- Requires custom implementations.                                                                                                                                                                                                                                                       |
| **Encoding Images in Various Formats** | \- **Save**: Create a file using **os.Create**, then use **png.Encoder** or **jpeg.Encoder** to encode and save the image.                                                                                                                                                                                                                                                                    | \- For other image formats like **WebP**, need to import the corresponding encoding package.                                                                                                                                                                                              |
| **Composing Image Layers**             | \- Does not directly support complex operations like combining images or drawing one image onto another.                                                                                                                                                                                                                                                                                      | \- Requires custom implementations.                                                                                                                                                                                                                                                       |
| **Resampling Methods**                 | \- Does not natively support advanced resampling techniques.                                                                                                                                                                                                                                                                                                                                  | \- Provides basic image manipulation capabilities, including resizing, but does not include advanced resampling algorithms.                                                                                                                                                               |
| **Saving the Image**                   | \- Does not directly support saving images or providing asynchronous save functionality.                                                                                                                                                                                                                                                                                                      | \- To perform asynchronous saving, need to use Go’s concurrency features such as goroutines.                                                                                                                                                                                              |

<div id="opencvsharp">

#### 8. OpenCvSharp

</div>

-   **Type**: Open-source

-   **Key Features**: .NET wrapper for OpenCV, similar to Emgu CV

-   **Licensing**: Free (Apache-2.0 license)

-   **Performance**: High performance, supports a wide range of tasks

-   **Integration Effort**: Moderate, different API style from Emgu CV

-   **Community and Support**: Active community, good documentation

-   **Restricted Countries contributors:** Need a check!

| **Feature Category**                   | **Supported by OpenCvSharp**                                                                                                                                                                                                                                           | **Not Natively Supported / Requires Custom Implementation**                           |
|:---------------------------------------|:-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:--------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by OpenCvSharp**                                                                                                                                                                                                                                           | **Not Natively Supported / Requires Custom Implementation**                           |
| **Image Loading and Creation**         | \- **Cv2.ImRead**, **Cv2.ImDecode**: Loads images from files or byte arrays. - **Mat**, **Cv2.ImCreate**: Creates new images with specified dimensions and types.                                                                                                      | \- No asynchronous loading methods like **Image.LoadAsync** in ImageSharp.            |
| **Image Processing and Manipulation**  | \- **Mat.Clone**, **Cv2.Resize**: For cloning and resizing. - **Cv2.CvtColor**: Converts between color spaces. - **Cv2.Rotate**, **Cv2.Flip**, **Cv2.WarpAffine**: Provides extensive geometric transformations. - **Cv2.Crop**: Crops images using a **Rect** object. | \- Lacks some of the more advanced image effects and filters available in ImageSharp. |
| **Pixel Formats**                      | \- Supports multiple formats, including **CvType.CV_8UC3** (BGR), **CvType.CV_8UC1** (Grayscale), **CvType.CV_8UC4** (BGRA). - Handles automatic color space conversion.                                                                                               | \- May require custom implementations for less common pixel formats.                  |
| **Pixel Access and Manipulation**      | \- **Mat.At**, **Mat.Set**: Direct pixel access and manipulation. - **Mat.Data**: Provides low-level access to pixel data arrays.                                                                                                                                      | \- Less intuitive pixel manipulation compared to ImageSharp’s **ProcessPixelRows**.   |
| **Image Metadata and Conversion**      | \- **Mat**: Stores image properties like size and type. - **Cv2.ImEncode**, **Cv2.ImWrite**: Converts and saves images in various formats.                                                                                                                             | \- Limited metadata handling compared to ImageSharp’s extensive metadata support.     |
| **Creating and Disposing Instances**   | \- **Mat**: Creates and initializes images. - **Dispose**: Required for freeing unmanaged resources. - Effective memory management through manual disposal.                                                                                                            | \- More complex resource management due to reliance on unmanaged resources.           |
| **Cropping and Resizing**              | \- **Cv2.GetRectSubPix**: For precise cropping. - **Cv2.Resize**: Resizing with various interpolation methods.                                                                                                                                                         | \- Lacks some of the more advanced cropping techniques like those in ImageSharp.      |
| **Encoding Images in Various Formats** | \- **Cv2.ImWrite**: Saves images in multiple formats including **WebP**. - **Cv2.ImEncode**: For encoding images to byte arrays.                                                                                                                                       | \- Less flexible format optimization compared to ImageSharp’s encoders.               |
| **Composing Image Layers**             | \- **Cv2.AddWeighted**, **Cv2.Add**, **Cv2.Subtract**: For image blending and compositing.                                                                                                                                                                             | \- Less comprehensive layering system than ImageSharp.                                |
| **Resampling Methods**                 | \- **Cv2.Resize**: Offers multiple interpolation methods. - **Cv2.PyrUp**, **Cv2.PyrDown**: Pyramid methods for scaling.                                                                                                                                               | \- Fewer options for specialized resampling methods compared to ImageSharp.           |
| **Saving the Image**                   | \- **Cv2.ImWrite**, **Cv2.ImEncode**: Saves images to files or byte arrays.                                                                                                                                                                                            | \- Lacks some advanced saving options like those in ImageSharp.                       |

<div id="microsoft.maui.graphics">

#### 9. Microsoft.Maui.Graphics

</div>

-   **Type**: Open-source

-   **Key Features**: Graphics functionalities across platforms, uses
    SkiaSharp

-   **Licensing**: Free (MIT)

-   **Performance**: Optimized for cross-platform use within MAUI
    framework

-   **Integration Effort**: Easy if using MAUI

-   **Community and Support**: Active support from Microsoft, good
    documentation

-   **Restricted Countries contributors:** No

This repository has been archived by the owner on Dec 21, 2023. It is
now read-only.

| **Feature Category**                   | **Supported by Microsoft.Maui.Graphics**                                                                                                                                                                                             | **Not Natively Supported / Requires Custom Implementation**                                                |
|:---------------------------------------|:-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:-----------------------------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by Microsoft.Maui.Graphics**                                                                                                                                                                                             | **Not Natively Supported / Requires Custom Implementation**                                                |
| **Image Loading and Creation**         | \- **IImageLoadingService.LoadImageAsync**: Asynchronous loading from various sources. - **GraphicsPlatform.CreateImage**: Creates images with specified dimensions and formats.                                                     | \- Unlike ImageSharp, there is limited support for creating images from scratch in more diverse formats.   |
| **Image Processing and Manipulation**  | \- **IImage.Clone**, **ICanvas.DrawImage**: For cloning, resizing, cropping, and applying transformations. - **ICanvas.SetFillColor**, **ICanvas.DrawRectangle**: Supports color transformations and selective drawing for cropping. | \- Lacks extensive image manipulation tools available in libraries like ImageSharp.                        |
| **Pixel Formats**                      | \- Supports **RGBA**, **RGB** and other common formats. - Designed for higher-level graphics tasks rather than extensive pixel format diversity.                                                                                     | \- Limited to basic pixel formats compared to the wide range in ImageSharp.                                |
| **Pixel Access and Manipulation**      | \- Focuses on high-level operations, without direct pixel access. - Does not offer low-level pixel manipulation like **GetPixel** and **SetPixel**.                                                                                  | \- No direct pixel access, unlike ImageSharp’s **ProcessPixelRows**.                                       |
| **Image Metadata and Conversion**      | \- Handles basic image properties and conversion via **IImage.Save**. - Can save to various formats.                                                                                                                                 | \- Less detailed metadata handling compared to ImageSharp.                                                 |
| **Creating and Disposing Instances**   | \- **GraphicsPlatform.CreateImage**: Easily creates new images. - Disposal managed via .NET’s garbage collection.                                                                                                                    | \- Lacks manual resource management options, unlike some lower-level libraries.                            |
| **Cropping and Resizing**              | \- **ICanvas.DrawImage**: Supports cropping and resizing with simple interfaces.                                                                                                                                                     | \- Less advanced cropping and resizing techniques compared to ImageSharp.                                  |
| **Encoding Images in Various Formats** | \- **IImage.Save**: Supports encoding in multiple formats. - Provides functionality to save directly to files, streams, or byte arrays.                                                                                              | \- Limited format-specific encoding settings compared to ImageSharp and does not mention **WebP** support. |
| **Composing Image Layers**             | \- **ICanvas.DrawImage**: Allows layering images over one another. - Basic support for image compositing and blending.                                                                                                               | \- More limited compositing features compared to ImageSharp.                                               |
| **Resampling Methods**                 | \- **ICanvas.DrawImage**: Automatically handles resampling during resizing.                                                                                                                                                          | \- Fewer advanced resampling methods compared to ImageSharp.                                               |
| **Saving the Image**                   | \- **IImage.Save**: Saves images to files, streams, or byte arrays in various formats.                                                                                                                                               | \- Lacks advanced saving options and optimizations available in ImageSharp.                                |

<div id="leadtools">

#### 10. LeadTools

</div>

-   **Type**: Commercial

-   **Key Features**: Extensive image processing features, supports
    numerous formats

-   **Licensing**: Paid (More than 17k$)

-   **Performance**: High performance, enterprise-grade reliability

-   **Integration Effort**: Moderate to high, depending on features used

-   **Community and Support**: Excellent support, extensive
    documentation

-   **Restricted Countries contributors:** Need a check!

| **Feature Category**                   | **Supported by LEADTOOLS**                                                                                                                                                                                                                                                          | **Not Natively Supported / Requires Custom Implementation**                                      |
|:---------------------------------------|:------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|:-------------------------------------------------------------------------------------------------|
| **Feature Category**                   | **Supported by LEADTOOLS**                                                                                                                                                                                                                                                          | **Not Natively Supported / Requires Custom Implementation**                                      |
| **Image Loading and Creation**         | \- **LoadImage, LoadAsync**: Robust image loading with asynchronous support. - **RasterImage.Create**: Flexible image creation with control over dimensions and pixel formats.                                                                                                      | \- None; LEADTOOLS provides comprehensive loading and creation features.                         |
| **Image Processing and Manipulation**  | \- **Clone, Resize, ConvertToGrayscale**: Supports advanced image processing tasks. - **Transformations, Annotations**: Extensive transformations and annotation capabilities. - **Image Enhancement**: High-level enhancement tools for noise reduction, contrast adjustment, etc. | \- While comprehensive, some specific manipulation tasks might require custom scripting or code. |
| **Pixel Formats**                      | \- Supports multiple formats: Extensive support for various pixel formats such as RGB, RGBA, Grayscale, CMYK, and more.                                                                                                                                                             | \- None; pixel format support is exhaustive.                                                     |
| **Pixel Access and Manipulation**      | \- **GetPixel, SetPixel, RasterImageData**: Direct access and manipulation of pixel data. - **Lock/Unlock**: For precise pixel-level operations.                                                                                                                                    | \- Pixel data manipulation can be complex, especially when locking/unlocking is required.        |
| **Image Metadata and Conversion**      | \- Comprehensive Metadata Handling: Extracts, edits, and writes metadata for a wide range of formats. - Advanced Conversion: Extensive format support including DICOM.                                                                                                              | \- None; LEADTOOLS excels in metadata and conversion capabilities.                               |
| **Creating and Disposing Instances**   | \- **CreateImage, Dispose**: Facilitates creation and proper resource disposal of images.                                                                                                                                                                                           | \- Proper disposal requires careful management, similar to .NET libraries.                       |
| **Cropping and Resizing**              | \- **Crop, Resize**: Advanced cropping and resizing with various resampling methods.                                                                                                                                                                                                | \- None; cropping and resizing are robustly supported.                                           |
| **Encoding Images in Various Formats** | \- **Save**: Encodes images into numerous formats including WebP, including advanced options for compression and format-specific settings. - Multimedia and Document Formats: Extends to multimedia and document encoding.                                                          | \- Format-specific settings might require additional configuration.                              |
| **Composing Image Layers**             | \- **Combine, DrawImage**: Supports complex image layering with detailed control over blending and transparency.                                                                                                                                                                    | \- Layer management might need additional custom code for complex use cases.                     |
| **Resampling Methods**                 | \- Advanced Resampling: Provides high-quality resampling techniques during resizing.                                                                                                                                                                                                | \- Resampling methods are extensive but need careful selection for optimal results.              |
| **Saving the Image**                   | \- **Save, SaveAsync**: Saves images with extensive control over parameters. - Asynchronous Saving: Supports performance-enhanced operations.                                                                                                                                       | \- None; saving features are comprehensive and powerful.                                         |
