# Definition and Rationale for the Metrics

##  Image Loading Time

**Definition:** Image loading time is defined as the total elapsed time
required to retrieve an image from a file system, decode it into a
standardized internal representation (e.g., RGBA32), and initialize any
associated data structures needed for further processing.

**Rationale:**

-   **I/O and Decoding Efficiency:** The time taken to load an image is
    a direct indicator of how efficiently a library handles input/output
    operations and decodes various image formats. In industrial
    applications, where images may be read in real time from cameras or
    sensors, fast loading is essential.

-   **Baseline for Processing Pipelines:** Since loading is the first
    step in any image processing pipeline, delays at this stage can
    cascade and affect overall system performance. An efficient image
    loading mechanism can significantly reduce the latency of the entire
    workflow.

-   **Minimizing Overhead:** The loading process also involves memory
    allocation and data structure initialization. A library that
    minimizes overhead in these areas will allow more resources to be
    devoted to the actual image processing tasks.

-   **Standardized Measurement:** The loading process can be uniformly
    measured across different libraries by using the same image datasets
    and environmental conditions, ensuring that the comparisons are fair
    and reproducible.

##  Pixel Iteration Time

**Definition:** Pixel iteration time refers to the duration required to
traverse every pixel in an image and apply a predefined, uniform
operation—such as converting an image to grayscale. This metric captures
the efficiency of the library’s core routines for low-level data
manipulation.

**Rationale:**

-   **Core Computation Performance:** Many image processing tasks (e.g.,
    filtering, thresholding, and color adjustments) require examining or
    modifying every pixel. The speed at which a library can iterate over
    pixels is a direct measure of its processing efficiency.

-   **Algorithmic Sensitivity:** Pixel-level operations are sensitive to
    implementation details like loop unrolling, memory access patterns,
    and caching strategies. Faster iteration times imply
    better-optimized routines.

-   **Simplicity and Reproducibility:** Converting an image to grayscale
    is a simple and commonly used operation that can serve as a proxy
    for other pixel-level tasks. Its simplicity makes it an ideal
    candidate for standardizing comparisons across libraries.

-   **Isolation of Low-Level Performance:** By isolating the pixel
    iteration operation from higher-level tasks, we can specifically
    evaluate the efficiency of the library’s data structures and
    internal algorithms without interference from more complex
    operations.
