# Image Conversion Benchmark Results

The image conversion benchmark was performed using ImageSharp and
Magick.NET as well as SkiaSharp and Structure.Sketching which were the
chosen libraries in their combinations with OpenCvSharp and Emgu CV,
respectively, for the conversion task. Using the same 4k resolution
image, the benchmark measured the time taken to convert the image from
JPEG to PNG format. Comparing the results of these libraries provides
insights into their performance and efficiency in application scenarios
where rapid image conversion is required—such as real-time image
processing pipelines or high-volume batch processing environments. The
data thus answer one of the central questions regarding which library
can provide significantly faster image conversion, thereby supporting
the hypothesis discussed in earlier chapters.

ImageSharp recorded an average conversion time of approximately 2,754
milliseconds. In contrast, the combination of OpenCvSharp with SkiaSharp
delivered an average conversion time of only 539 milliseconds.
Similarly, Emgu CV integrated with Structure.Sketching achieved an
average time of 490 milliseconds, while Magick.NET registered an average
conversion time of 4,333 milliseconds.

<div id="tab:image-conversion-results">

| **Library**                       | **Warm-Up Time (ms)** | **Avg. Time Excl. Warm-Up (ms)** | **Total Time Incl. Warm-Up (ms)** |     |
|:----------------------------------|:----------------------|:---------------------------------|:----------------------------------|:----|
| **Library**                       | **Warm-Up Time (ms)** | **Avg. Time Excl. Warm-Up (ms)** | **Total Time Incl. Warm-Up (ms)** |     |
| **ImageSharp**                    |                       |                                  |                                   |     |
| **OpenCvSharp + SkiaSharp**       |                       |                                  |                                   |     |
| **Magick.NET**                    |                       |                                  |                                   |     |
| **Emgu CV + Structure.Sketching** |                       |                                  |                                   |     |
|                                   |                       |                                  |                                   |     |

The Image Conversion Benchmark Results in milliseconds, showing the
warm-up time, average time excluding warm-up, and total time including
warm-up for each library or combination.

</div>

The table
<a href="#tab:image-conversion-results" data-reference-type="ref" data-reference="tab:image-conversion-results">1</a>,
is the final dataset that been constructed by merging multiple Excel
files produced by the framework described in the Implementation chapter.
These results shows lightweight libraries such as SkiaSharp and
Structure.Sketching outperforming ImageSharp and Magick.NET in terms of
image conversion time. The data also reveals that Emgu CV with
Structure.Sketching is the most efficient combination for image
conversion, with the lowest average time of 490 milliseconds. On the
other hand, ImageSharp and Magick.NET are significantly slower, with
average times of 2,754 and 4,333 milliseconds, respectively.

To visually summarize these findings, Figure
<a href="#fig:image-conversion" data-reference-type="ref" data-reference="fig:image-conversion">[fig:image-conversion]</a>
presents a bar chart that depicts the conversion times across the
evaluated libraries. The graph clearly demonstrates that the conversion
times for the OpenCvSharp+SkiaSharp and Emgu CV+Structure.Sketching
combinations are positioned at the lower end of the performance
spectrum, while ImageSharp exhibits considerably higher times. A
logarithmic scale has been employed to effectively represent the
significant differences in total times—comprising both the warm-up
periods and the average conversion times. This three-color graphical
representation enables a thorough comparison of library performance in
various contexts, such as real-time image processing and batch
conversion tasks, thereby reinforcing the quantitative analysis
presented earlier.

<div class="center">

<img src="media/log_1.png" style="width:5in" alt="image" /> <span
id="fig:image-conversion"
label="fig:image-conversion">\[fig:image-conversion\]</span>

</div>
