# Library Selection Criteria

## Selection Process

The libraries were chosen based on a combination of performance
benchmarks, feature sets, and cost considerations. The selection process
involved the following steps:

-   **Initial Survey:** A comprehensive review of available image
    processing libraries in the .NET and cross-platform ecosystems was
    conducted.

-   **Feature Mapping:** Each library was evaluated for core
    functionalities such as image loading, pixel manipulation, support
    for multiple pixel formats, and the ability to perform complex
    operations (e.g., cropping, resizing, and layer composition).

-   **Cost Analysis:** Given that ImageSharp carries a recurring
    licensing cost, alternatives were considered if they offered similar
    capabilities at a lower or no cost. This factor was critical for
    long-term sustainability.

## Criteria for Comparison

The following criteria were used to compare and select libraries:

-   **Performance:** Measured via the defined metrics (image conversion
    and pixel iteration times). Libraries with faster and more
    consistent performance were favored.

-   **Functionality:** The library’s ability to handle a broad spectrum
    of image processing tasks. This includes both low-level operations
    (like direct pixel access) and high-level operations (like image
    composition).

-   **Ease of Integration:** Libraries that offered simple integration
    with the .NET framework (or had comprehensive wrappers) were
    prioritized. The ease of adoption and the availability of community
    support or documentation were also key considerations.

-   **Cost and Licensing:** Free and open-source libraries were
    preferred. However, if a commercial library provided substantial
    performance or feature benefits—justifying its cost—it was also
    considered.

-   **Scalability and Maintainability:** Future scalability was taken
    into account. A library that could efficiently handle larger images
    or more complex processing tasks was seen as more future-proof.

## Rationale for Criteria

The selection criteria were chosen to ensure that the chosen libraries
would meet the following key requirements:

-   **Performance and Functionality:** At the heart of image processing
    is the ability to quickly and efficiently manipulate image data. The
    chosen metrics directly reflect these capabilities. Any library that
    underperformed in these areas would risk impacting the overall user
    experience.

-   **Ease of Integration and Cost:** For both academic and practical
    applications, it is important that the chosen solution does not
    require extensive re-engineering or incur significant ongoing costs.
    Libraries that can be integrated with minimal effort and without
    additional licensing fees are therefore more attractive.

-   **Scalability:** As image resolutions continue to increase and
    applications demand more real-time processing, scalability becomes a
    critical factor. Libraries with proven performance in handling large
    datasets are better suited for future challenges.
