# Measurement Procedure

### Experimental Setup

Two sets of benchmark tests were developed:

-   **Image Conversion Benchmark:**

    -   **Process:** The test reads an image from disk, performs a
        format conversion (reading a JPG and writing a PNG), and then
        saves the output.

    -   **Measurement:** A high-resolution timer (the ‘Stopwatch‘ class
        in .NET) is used to record the elapsed time for each operation.

    -   **Iterations:** A series of 100 iterations is executed, preceded
        by a number of warm-up iterations to mitigate the effects of
        just-in-time compilation and caching.

-   **Pixel Iteration Benchmark:**

    -   **Process:** The image is loaded, and every pixel is accessed
        sequentially. For each pixel, a simple grayscale conversion is
        applied.

    -   **Measurement:** Similar to the conversion test, the time taken
        for each iteration is recorded using a high-resolution timer.

    -   **Iterations:** Again, a fixed number of warm-up iterations is
        used before measuring the main iterations.

### Data Collection and Processing

-   **Warm-Up and Main Iterations:**

    -   Warm-up iterations are run to stabilize the runtime environment.
        Their durations are recorded separately to ensure that only
        steady-state performance is analyzed.

    -   The main iterations are then executed, and the time for each
        iteration is recorded.

-   **Metrics Computed:**

    -   **Warm-Up Time:** Total time consumed during warm-up iterations.

    -   **Average Time Excluding Warm-Up:** The mean duration of the
        main iterations, providing a normalized measure of performance.

    -   **Total Time Including Warm-Up:** The cumulative duration that
        includes both warm-up and main iterations.

-   **Result Storage:**

    -   The results are written to an Excel file using a library like
        EPPlus, which facilitates further analysis and visualization.
        This systematic storage allows for easy comparison across
        different libraries.
