# Memory Benchmarking Results

In parallel with the time benchmarks, memory consumption was a critical
parameter in the evaluation. For the image conversion tasks, SkiaSharp,
as part of the OpenCvSharp+SkiaSharp configuration, exhibited the lowest
memory allocation, with values approximating 58 KB. ImageSharp, in
comparison, required about 5.67 MB, which is substantially higher. In
the context of pixel iteration, the memory profiles were similarly
divergent. ImageSharp was extremely efficient in this regard, consuming
roughly 20 KB on average, whereas Emgu CV + Structure.Sketching, that
performed exceptionally well in terms of speed for pixel iteration, in
memory terms, was less efficient. It consumed around 170 MB of memory,
which is significantly higher than the other libraries tested.
SkiaSharp,

<div id="tab:memory-results-image-conversion">

| **Library**    | **Allocated Memory** | **Gen0/Gen1/Gen2 Collections** |     |     |
|:---------------|:---------------------|:-------------------------------|:----|:----|
| **Library**    | **Allocated Memory** | **Gen0/Gen1/Gen2 Collections** |     |     |
| **EmguCV**     | MB (712 bytes)       | \- / - / -                     |     |     |
| **ImageSharp** | MB (5,805.41 KB)     | ,000 / 1,000 / 1,000           |     |     |
| **SkiaSharp**  | MB (58,864 bytes)    | \- / - / -                     |     |     |
|                |                      |                                |     |     |

Memory benchmarking results for the image conversion task, detailing the
allocated memory (in MB) along with the associated Gen0, Gen1, and Gen2
garbage collection counts.

</div>

The table
<a href="#tab:memory-results-image-conversion" data-reference-type="ref" data-reference="tab:memory-results-image-conversion">1</a>
summarizes the memory benchmarking results for image conversion. It is
evident that ImageSharp has the highest memory allocation, with
approximately 5.67 MB, while SkiaSharp has the lowest. Emgu CV falls in
between, with a memory allocation of 0.00068 MB. These figures provide a
clear indication of the memory efficiency of each library for image
conversion tasks. Garbage collection counts are also included to provide
additional context on the memory management behavior of each library.
Gen0, Gen1, and Gen2 collections means the number of times each
generation was collected during the benchmarking process. These metrics
are essential for understanding how each library manages memory and how
it impacts performance.

The large memory footprint of Emgu CV during pixel iteration is a
noteworthy trade-off. While its performance in terms of speed is
excellent, the high memory consumption must be considered when deploying
the solution in memory-constrained environments. The benchmarking data
collected here is critical because it provides a balanced view—speed
alone does not define an optimal library, but rather the ratio of
processing time to memory usage does. For a clear summary of these
findings, table
<a href="#tab:memory-results-pixel-iteration" data-reference-type="ref" data-reference="tab:memory-results-pixel-iteration">2</a>
provides a concise overview of the memory metrics for each library
configuration.

<div id="tab:memory-results-pixel-iteration">

| **Library**    | **Allocated Memory**   | **Gen0/Gen1/Gen2 Collections** |     |     |
|:---------------|:-----------------------|:-------------------------------|:----|:----|
| **Library**    | **Allocated Memory**   | **Gen0/Gen1/Gen2 Collections** |     |     |
| **EmguCV**     | MB (177,976,185 bytes) | ,142 / 1,571 / 1,571           |     |     |
| **ImageSharp** | MB (20.26 KB)          | \- / - / -                     |     |     |
| **SkiaSharp**  | MB (403,300,552 bytes) | / - / -                        |     |     |
|                |                        |                                |     |     |

Memory Benchmarking Results for Pixel Iteration Task, detailing the
allocated memory (in MB) along with the associated Gen0, Gen1, and Gen2
garbage collection counts.

</div>

The table
<a href="#tab:memory-results-pixel-iteration" data-reference-type="ref" data-reference="tab:memory-results-pixel-iteration">2</a>
indicates that while SkiaSharp has the highest memory allocation for
pixel iteration of approximately 384 MB, ImageSharp is the most
memory-efficient, with a memory allocation of 0.01932 MB. Emgu CV falls
in between, with a memory allocation of 170 MB. These figures provide a
clear indication of the memory efficiency of each library for pixel
iteration tasks. Garbage collection counts are also included to provide
additional context on the memory management behavior of each library.
Gen0, Gen1, and Gen2 collections means the number of times each
generation was collected during the benchmarking process. This means
that the garbage collector had to run 33,142 times for Gen0, 1,571 times
for Gen1, and 1,571 times for Gen2.
