# Overview

Image processing tasks in industrial applications typically involve two
fundamental operations: acquiring image data (loading) and performing
pixel-level computations (iteration). The efficiency of these operations
directly influences the overall performance of any image processing
system. In our evaluation, we have chosen to focus on two key metrics:

-   **Image Loading Time:** The time taken to load an image from
    persistent storage into the system’s memory.

-   **Pixel Iteration Time:** The duration required to traverse and
    process each pixel in an image, exemplified by converting the image
    to grayscale.

These metrics are chosen for several reasons:

-   **Universality:** Both operations are common to nearly all image
    processing workflows, regardless of the complexity of the subsequent
    processing steps.

-   **Fundamental Performance Indicators:** The speed of image loading
    reflects the efficiency of file I/O, image decoding, and memory
    allocation, while pixel iteration performance indicates how well a
    library can handle low-level data manipulation—an operation that is
    central to filtering, enhancement, and other pixel-based
    computations.

-   **Comparability and Reproducibility:** By using standardized tasks
    that all libraries must perform, we can compare their performance on
    a like-for-like basis. This approach minimizes variability and
    provides a clear baseline for comparing otherwise diverse systems.

-   **Hardware Independence:** These metrics are less influenced by
    high-level algorithmic choices and more by the underlying
    implementation and optimizations, making them suitable for
    benchmarking across different libraries and platforms.

While there are many other potential metrics (such as encoding time,
advanced filtering speed, or transformation accuracy), we selected image
loading and pixel iteration because they are both critical and
universally applicable operations. They provide a controlled environment
for performance measurement and are directly relevant to the low-level
efficiency needed in industrial scenarios.
