# Related Work

The evaluation of image processing libraries, particularly for
industrial applications, has attracted significant research interest
over the past decades. Broadly, the field encompasses automated image
analysis and computer vision systems designed to handle tasks such as
quality control, defect detection, and high-resolution image
enhancement. The foundational research in Automated Image Processing has
evolved from early, often ad hoc, implementations to sophisticated
frameworks that leverage hardware acceleration and advanced algorithms.
Early surveys, such as Kulpa’s (1981) seminal review of digital image
processing systems in Europe, laid the groundwork for understanding the
challenges of standardization and performance evaluation in these
systems.

In recent years, the convergence of hardware acceleration and image
analysis has been a recurring theme. Sahebi et al. (2023) demonstrate
how distributed processing on FPGAs can dramatically enhance
computational efficiency—a principle equally applicable to industrial
image processing where real-time performance is critical. Similarly, Ma
et al. (2024) contribute to the field by presenting an image quality
database specifically tailored for industrial processes. Their work
emphasizes the importance of aligning objective metrics with human
perception in quality assessments, a concern that resonates throughout
subsequent research in the area.

Chisholm et al. (2020) and Ferreira et al. (2024) extend these
discussions by focusing on the implementation of real-time image
processing systems using FPGAs. Chisholm illustrate a real-time crack
detection system employing particle filters, highlighting the challenges
of meeting stringent timing constraints in industrial settings.
Ferreira, on the other hand, propose a generic FPGA-based pre-processing
library, emphasizing strategies to minimize memory overhead and improve
processing speed. These studies underscore the significant role of
hardware acceleration in modern image processing pipelines, setting the
stage for more nuanced comparative evaluations.

A critical aspect of the research is the comparative analysis of
different image processing libraries. Lai et al. (2001) provide an
in-depth review of several libraries, contrasting hardware-specific
optimizations with generic, portable solutions. Their work not only
identifies the strengths and weaknesses inherent in different design
philosophies but also serves as a benchmark against which later
approaches can be compared. Kulpa’s early survey (1981) remains an
important historical reference, offering insights into the evolution of
image processing systems and highlighting persistent issues such as
limited standardization and documentation.

Pérez et al. (2014) contribute by investigating super-resolution
techniques for plenoptic cameras via FPGA-based implementations,
demonstrating that hardware acceleration can significantly improve both
processing speed and image quality. Meanwhile, Rao’s (2023) comparative
analysis of deep learning frameworks extends the conversation by
incorporating performance metrics, documentation quality, and community
support. This approach is particularly valuable as it parallels the
metrics used to evaluate traditional image processing libraries, thereby
bridging the gap between classical image processing and modern deep
learning paradigms.

Several studies have explored niche industrial applications where image
processing plays a critical role. Ciora and Simion (2014) provide a
broad overview of the applications of image processing in industrial
engineering, covering areas from automated visual inspection to process
control. Their comprehensive review underscores the necessity of robust,
efficient image processing systems that integrate seamlessly with
industrial control mechanisms.

In a more focused domain, Sandvik et al. (2024) review machine learning
and image processing techniques for wood log scaling and grading. Their
systematic categorization of methodologies offers a template for
benchmarking approaches that combine computer vision with
domain-specific performance metrics. Sardar (2012) examines the use of
image processing for quality analysis in agriculture, further
highlighting the versatility of these technologies across different
industrial sectors.

Vieira et al. (2024) address the challenges of deploying image
processing algorithms on Programmable Logic Controllers (PLCs), which
are prevalent in industrial control systems. Their work illustrates the
trade-offs between processing speed, implementation complexity, and
system robustness when operating in resource-constrained environments.
Wu et al. (2022) and Zhu et al. (2022) then delve into specific
industrial applications—precision control in filament drafting and
product appearance quality inspection, respectively—demonstrating the
critical impact of real-time processing and integration on system
performance.

At the forefront of current research are studies that provide robust
benchmarking frameworks. Reis (2023) offers an overview of recent
developments in computer vision and image processing methodologies,
pointing out the increasing integration of artificial intelligence with
classical approaches. This evolution is complemented by Ziaja et al.
(2021) , whose work on benchmarking deep learning for on-board space
applications provides a rigorous framework for evaluating execution
time, resource utilization, and overall performance under constrained
hardware conditions.

These contemporary evaluations are essential for highlighting the
limitations of existing approaches. While many studies focus on
performance metrics such as processing speed and memory efficiency, few
have systematically integrated these factors with ease of integration
and system robustness in industrial settings. This gap in the literature
motivates the present study, which aims to establish a comprehensive
benchmarking approach that encompasses both hardware acceleration and
software flexibility.

In summary, the reviewed literature presents a rich tapestry of
methodologies and evaluations that span a broad spectrum of industrial
image processing applications. Early foundational works provided
historical context and identified critical challenges, while subsequent
studies advanced the field by integrating hardware acceleration, deep
learning, and niche industrial applications into comprehensive
performance evaluations. Despite these advances, a clear gap remains in
the standardization of benchmarking protocols that address performance,
resource efficiency, and integration challenges in real-world industrial
settings. This thesis proposes a novel benchmarking approach that
differentiates itself by not only comparing the computational
performance of various image processing libraries but also by evaluating
their ease of integration into complex industrial workflows. By doing
so, the study seeks to provide actionable insights for practitioners and
pave the way for the next generation of robust, efficient, and versatile
image processing solutions.
