#  Research Questions and Investigative Focus

In this section, the core questions that guided the research in this
master thesis are examined. Rather than adopting a traditional
hypothesis-driven approach, the study focused on a systematic, empirical
evaluation of image processing libraries. The investigation was centered
on two main questions:

1.  What is the performance of different libraries when executing a
    defined set of image processing tasks?

2.  Which library delivers the most resource-efficient processing?

While the nature of the research did not lend itself to a single,
testable hypothesis, the work instead embraced an exploratory and
comparative framework to assess each library’s strengths and
limitations.

##  Defining the Research Questions

At the outset, the investigation was framed by two primary research
questions. The first sought to understand the performance
characteristics of various image processing libraries when applied to
common tasks. In this context, “performance” encompassed both execution
time and memory usage. Specifically, the study measured how efficiently
each library could handle tasks such as image loading, format conversion
(for example, converting a JPEG image to PNG), and pixel-level
operations like iterating over and converting pixels to grayscale.

The second research question aimed to identify which library exhibited
the highest resource efficiency. Resource efficiency in this study was
defined not only by the speed of processing but also by the amount of
memory allocated during image manipulation operations. The investigation
compared several leading libraries—including, but not limited to,
OpenImageIO, SkiaSharp, Magick.NET, Emgu CV, and MagicScaler—to
determine which provided the most favorable balance of performance and
memory usage.

It is important to note that, while a hypothesis was initially
considered, the research context—centered on performance benchmarking
and resource analysis—meant that a traditional hypothesis (i.e., a
prediction that one library would outperform others under all
conditions) was not easily applicable. Instead, the study was designed
as an exploratory evaluation to chart the multifaceted performance
landscape of image processing libraries.

##  Methodology and Performance Metrics

To address these research questions, the investigation was conducted
through a series of carefully designed experiments. Two core tests
formed the backbone of the methodology: the image conversion test and
the pixel iteration test.

###  Image Conversion Test

The image conversion test was designed to measure the time required by
each library to load an image, convert it to a different format, and
save the converted image to disk. This test was representative of common
real-world workflows where images are manipulated for different output
requirements. The process involved several steps:

-   **Loading the Image:** Each library’s ability to read an image from
    memory was assessed. The test considered how quickly a library could
    load an image file into its internal data structures.

-   **Conversion Process:** Once the image was loaded, the library was
    tasked with converting the image from one format (for example, JPEG)
    to another (such as PNG). This conversion process tested the
    library’s efficiency in handling image encoding and decoding.

-   **Saving the Converted Image:** Finally, the time taken to save the
    converted image back to disk was recorded. This step provided a
    complete view of the end-to-end processing time.

By repeating this process across multiple libraries and averaging the
results, the study was able to generate comparative performance metrics.
For instance, preliminary findings showed that libraries such as
SkiaSharp demonstrated remarkably fast conversion times, accompanied by
minimal memory usage during the process.

###  Pixel Iteration Test

The second experiment focused on the pixel iteration test. This test
evaluated how long it took each library to perform a pixel-by-pixel
operation on an image—a task common in many image processing
applications such as filtering or applying color transformations (e.g.,
converting an image to grayscale). The key steps in this test included:

-   **Loading the Image:** As in the conversion test, the first step was
    to load an image into memory.

-   **Pixel Operation:** The library was then tasked with iterating over
    each pixel in the image and applying a simple transformation. In
    this case, the chosen operation was converting the pixel values to
    grayscale.

-   **Timing the Process:** The total time taken to complete the
    operation across all pixels was recorded. This metric was crucial
    for assessing the efficiency of the library’s pixel-level processing
    capabilities.

-   **Memory Allocation:** In addition to execution time, the test
    tracked the memory consumption during the pixel iteration process.
    This provided insight into how resource-intensive each library was
    when performing granular image manipulations.

By analyzing the results from both tests, the research provided a dual
perspective on performance: one focused on throughput (how fast
operations are completed) and the other on resource utilization (how
much memory is consumed during processing).
