# Result Export and Data Aggregation

Once the performance and memory metrics were collected, the next
challenge was to present the results in a coherent and accessible
manner. For this purpose, Excel was chosen as the output format due to
its widespread adoption and ease of use for further analysis.
`OfficeOpenXml` namespace, which is part of the EPPlus library, allows
for the creation and manipulation of Excel files in .NET applications.
The ExcelExporter class was implemented to aggregate the benchmark
results and export them to an Excel file.

The code snippet below illustrates how the benchmark results are
aggregated and exported to an Excel file:

``` [Sharp]C
using OfficeOpenXml;

public class ExcelExporter
{
    public static void ExportResults(string excelOutputPath, 
        (double warmupTime, double averageTime, double totalTime) imageConversionResults, 
        (double warmupTime, double averageTime, double totalTime) pixelIterationResults)
    {
        using (var package = new ExcelPackage())
        {
            var worksheet = package.Workbook.Worksheets.Add("Benchmark Results");
            worksheet.Cells[1, 1].Value = "Benchmark";
            worksheet.Cells[1, 2].Value = "Warm-Up Time (ms)";
            worksheet.Cells[1, 3].Value = "Average Time (ms)";
            worksheet.Cells[1, 4].Value = "Total Time (ms)";
            
            worksheet.Cells[2, 1].Value = "Image Conversion";
            worksheet.Cells[2, 2].Value = imageConversionResults.warmupTime;
            worksheet.Cells[2, 3].Value = imageConversionResults.averageTime;
            worksheet.Cells[2, 4].Value = imageConversionResults.totalTime;
            
            worksheet.Cells[3, 1].Value = "Pixel Iteration";
            worksheet.Cells[3, 2].Value = pixelIterationResults.warmupTime;
            worksheet.Cells[3, 3].Value = pixelIterationResults.averageTime;
            worksheet.Cells[3, 4].Value = pixelIterationResults.totalTime;
            
            package.SaveAs(new FileInfo(excelOutputPath));
        }
    }
}
```

The ExcelExporter class creates a structured Excel file with separate
sheets for each benchmark operation. The results are organized into
columns for the warm-up time, average time, and total time for each
operation. The resulting Excel file provides a clear and concise summary
of the benchmark results, making it easy to compare the performance and
memory characteristics of each library.

By automating the process of result aggregation, the framework not only
saves time but also minimizes the risk of manual errors. Each cell in
the generated Excel file is carefully populated with benchmark data, and
the resulting spreadsheet can be easily imported into analytical tools
for further exploration. This process of exporting results serves as a
bridge between the raw performance data and the actionable insights that
drive decision-making in software optimization.
