## Benchmarking Overview:

In this benchmark, we utilized BenchmarkDotNet to measure the
performance of various image processing libraries. BenchmarkDotNet is a
powerful .NET library that provides accurate and detailed performance
metrics. It handles the complexities of benchmarking, such as warm-up,
iteration, and statistical analysis, ensuring reliable results.

The benchmarking process involved the following steps:

-   **Setup:** Initialize the environment and load the image.

-   **Warm-Up:** Perform initial iterations to stabilize the
    environment.

-   **Measurement:** Execute the image processing tasks (Image
    Conversion and Pixel Iteration) and record the time and memory
    usage.

-   **Analysis:** Analyze the collected data to determine the average,
    total, and warm-up times, as well as memory allocations and garbage
    collections.

The attached images and tables provide a detailed comparison of the
performance metrics for different libraries, highlighting their
strengths and weaknesses in various tasks.

## Benchmark Results:

<img src="media/image5.png" style="width:5.90069in;height:2.88858in" alt="image" />

| **Library**    | **Task**         | **Mean Time** | **Allocated Memory** | **Gen0/Gen1/Gen2 Collections** |
|:---------------|:-----------------|:--------------|:---------------------|:-------------------------------|
| **Library**    | **Task**         | **Mean Time** | **Allocated Memory** | **Gen0/Gen1/Gen2 Collections** |
| **EmguCV**     | Image Conversion | ms            | MB (712 bytes)       | \- / - / -                     |
| **ImageSharp** | Image Conversion | ms            | MB (5,805.41 KB)     | ,000 / 1,000 / 1,000           |
| **SkiaSharp**  | Image Conversion | ms            | MB (58,864 bytes)    | \- / - / -                     |

<img src="media/image6.png" style="width:5.90069in;height:2.88858in" alt="image" />

| **Library**    | **Task**        | **Mean Time** | **Allocated Memory**   | **Gen0/Gen1/Gen2 Collections** |
|:---------------|:----------------|:--------------|:-----------------------|:-------------------------------|
| **Library**    | **Task**        | **Mean Time** | **Allocated Memory**   | **Gen0/Gen1/Gen2 Collections** |
| **EmguCV**     | Pixel Iteration | ms            | MB (177,976,185 bytes) | ,142 / 1,571 / 1,571           |
| **ImageSharp** | Pixel Iteration | ms            | MB (20.26 KB)          | \- / - / -                     |
| **SkiaSharp**  | Pixel Iteration | s             | MB (403,300,552 bytes) | / - / -                        |
